function out = fit_ssrga_params(Dmax, Araw, dipole_dx, gamma,...
			       rescale_each_x)
  % FIT_SSRGA_PARAMS
  %   out = fit_ssrga_params(Dmax, Araw, dipole_dx, gamma,rescale_each_x)
  % This function fits the parameters of the Self-Similar
  % Rayleigh-Gans Approximation. The input "Araw" is a cell array of
  % vectors that represent the unnormalized mass distribution through
  % np=length(Dmax) particles, where Dmax is the maximum dimension of
  % each particle in metres and dipole_dx is the separation of the
  % points in the Araw vectors, also in metres. The length of Araw
  % must be a multiple of np, and it is assumed that there are ns =
  % length(Araw)/np samples per particle, with Araw{1:ns} representing
  % samples of particle 1, Araw{ns+1:2*ns} representing samples of
  % particle 2 and so on. Each particle is normalized such that it has
  % the same mass and mean physical extent. The power spectrum is
  % fitted with the specified gamma factor.
  % The final argument "rescale_each_x" is optional and provides
  % similar behaviour to the original fitting algorithm: the x axis of
  % each Araw entry is rescaled to lie between -0.5 and 0.5. This
  % means that the wavelength of the structure is changed, which is
  % not ideal, so this option is not the default. 

if nargin < 5
  rescale_each_x = 0;
end

% Number of vectors
nA = length(Araw);

% Number of particles
np = length(Dmax);

% Maximum dimension in units of dipoles
Dmax_dipoles = Dmax ./ dipole_dx;

if mod(nA,np) ~= 0
  error 'length(Araw) must be a multiple of np'
end

% Number of samples per particle
ns = nA/np;

% We wish to scale each vector such that it has a mean of 1, and work
% out a spatial coordinate scale "x" such that the average spatial
% extent of the particles is 1. Each is shifted so that its centre of
% mass is at x=0.  We then interpolate onto a common axis "X":
nX = 1024;
X = linspace(-1,1,nX);

% Compute the length of each vector
for iA = 1:nA
  nx(iA) = length(Araw{iA});
end
% Compute the mean length of each vector for each particle
mean_nx = mean(reshape(nx, ns, np),1);
%mean_nx = max(reshape(nx, ns, np));

% Effective aspect ratio is the ratio of the mean extent of a particle
% to the maximum dimension
out.alpha_eff = mean(mean_nx ./ Dmax_dipoles);
out.alpha_eff = 0.01.*round(out.alpha_eff.*100);

% Reshape mean_nx so that there is one entry per element of Araw.
mean_nx = ones(ns,1)*mean_nx;
mean_nx = mean_nx(:);

% Normalize each sample and insert into matrix A
A = zeros(nA,nX);
index = find(X>=-0.5 & X<0.5);
for iA = 1:nA
  if ~rescale_each_x
    % Temporary spatial coordinate such that on average each particle
    % has a spatial extent of 1
    x = linspace(0,nx(iA)./mean_nx(iA),nx(iA));
    % Offset so that the centre of gravity is at x=0
    x = x - sum(x.*Araw{iA})./sum(Araw{iA});
  else
    % Older method where x is rescaled to lie between -0.5 and 0.5 for
    % each A function
    x = linspace(-0.5, 0.5, nx(iA));
  end
  % Interpolate onto common grid, with zeros outside
  A_full = interp1(x,Araw{iA},X,'linear',0);
  % Insert normalized version into matrix A, where the normalization
  % is such that the mean value in the range -0.5 to 0.5 is equal to
  % 1.
  A(iA,:) = A_full ./ mean(A_full(index));

  % Store particle volume
  vol(iA) = sum(Araw{iA}).*dipole_dx.^3;
end

% Store mean volume
out.volume = mean(vol);

% Store the variables need to plot the A functions, including their
% mean and standard deviation
out.X = X;
%out.A = A; % Rather big to store
out.meanA = mean(A,1);
out.stdA = std(A);

% Compute the kurtosis parameter "kappa" - first compute the kappa
% look-up table as a function of spatial variance
[kappas_lut, Avar_lut] = calc_kappa_lut;
% Compute the spatial variance
Avar = mean(X(index).^2.*out.meanA(index));
% Interpolate to find kappa
out.kappa = interp1(Avar_lut,kappas_lut,Avar,'nearest');

% Compute the fitted A function in the range -0.5 < X < 0.5
Xfit = X(index);

% The following has a mean of 1 so no need to normalize
Afit = 0.5.*pi.*((1+out.kappa/3).*cos(pi.*Xfit)+out.kappa.*cos(3.*pi.*Xfit));
% Not needed
%Afit = Afit ./ mean(Afit);

out.Xfit = Xfit;
out.Afit = Afit;

% Compute the difference between the A functions and the fit. 
Adiff =A(:,index) - ones(nA,1)*Afit;

% Compute average power spectrum
nXfit = length(index);
sum_power_spectrum = zeros(1,nXfit./2);
sum_log_power_spectrum = zeros(1,nXfit./2);
for iA = 1:nA
  F = fft(Adiff(iA,:));
  nF = length(F);
  % Compute power spectrum
  E = F.*conj(F).*2.0/(nF.*nF);
  % Ignore first element: represents the mean of the field
  sum_power_spectrum = sum_power_spectrum + E(2:nXfit./2+1);
  sum_log_power_spectrum = sum_log_power_spectrum + log(E(2:nXfit./2+1));
end
% Compute average power spectrum
power_spectrum = sum_power_spectrum ./ nA;
%power_spectrum = exp(sum_log_power_spectrum ./ nA);

% Save power spectrum and it's coordinate "j"
out.j = 1:nXfit./2;
out.power_spectrum = power_spectrum;

% Check Parseval's theorem holds
out.variance_ratio_SHOULD_BE_ONE = sum(power_spectrum)./var(Adiff(:));
out.variance_obs = var(Adiff(:));
% Compute "beta" of fitted power law such that the power law matches
% the integral of the power spectrum for the following elements.  Note
% that we don't fit the first element since it is always below the
% fitted power law
index_largescale = 2:12;
%index_largescale = 3:12;

variance_largescale = sum(power_spectrum(index_largescale));
power_law_fit = (2.*out.j).^-gamma;
% Since the sin & cos terms are proportional to sqrt(beta), but the
% variance of sin(x) is 0.5, we need to multiply the variance by 2 to
% get beta. Moreover, in the case of kappa=0, we want the amplitude of
% the first cosine term of Afit to be 1, so that the remaining
% power-spectrum terms are relative to that and beta is scaled
% correctly, so we multiply further by (2/pi)^2:
beta = (8/pi.^2).*variance_largescale./sum(power_law_fit(index_largescale));
power_law_fit = beta.*power_law_fit./(8./pi.^2);
% Save beta, gamma and the ratio of the first element to the power-law
% fit
out.first_element_ratio = 0.01*round(100.*power_spectrum(1) ./ power_law_fit(1));
out.beta = 0.01.*round(beta.*100);
%out.beta = beta;
out.gamma = gamma;
% Save the power-law fit, reducing the first element to the data
power_law_fit(1) = power_spectrum(1);
out.power_law_fit = power_law_fit;

