function bcs = rayleigh_gans_spheroid(wavelength, dielectric_const, ...
                                      D, volume)
%RAYLEIGH_GANS_SPHEROID  
%   Compute the backscatter cross section of a spheroid using the
%   Rayleigh-Gans Approximation
%
%   bcs = rayleigh_gans_spheroid(wavelength, dielectric_const, ...
%                                D, volume)
%   where the arguments are:
%     bcs                  Backscatter cross-section (m2)
%     wavelength           Wavelength of radiation (m)
%     dielectric_const     Dielectric constant of material (complex)
%     D                    Particle size in direction of propagation (m)
%     volume               Volume of material (m3)
%
%   All arguments may be vectors or scalars, provided that any vectors are
%   the same length.
%
%   This file was written by Robin Hogan, but no copyright is asserted: this
%   file is in the public domain.  Therefore, copying and distribution of
%   this file, with or without modification, and merging all or part of this
%   file into other works, are permitted in any medium without royalty.
%   This file is offered as-is, without any warranty.

% Dielectric factor
K2 = abs(((dielectric_const-1)./(dielectric_const+2))).^2;

% Wavenumber
k = 2.*pi./wavelength;

% Prefactor containing volume and dielectric-constant dependence
prefactor = (81/4).*K2.*volume.*volume./(pi.*k.*k.*D.^6);

% Shape dependence
shape_term = sin(k.*D) - k.*D.*cos(k.*D);

% Compute backscatter cross section
bcs = prefactor.*shape_term.*shape_term;

index = find(D == 0.0);
if ~isempty(index)
  bcs(index) = (9./4).*K2.*volume(index).*volume(index).*k.^4./pi;
end

