% Script to test fit_ssrga_params by generating synthetic A(s)
% functions to feed to it

% In the fitting procedure, do we use the older method of rescaling
% the x axis of each A function in order to lie between x=-0.5 and 0.5
% (rescale_each_x=1)? Or do we use the same x scale for all A
% functions, but shift each one so that its centre of gravity lies at
% x=0 (rescale_each_x=0)?  A value of 1 is needed in the present
% script in order to check that the fitted beta matches the prescribed
% one, because the act of shifting each A so that its centre of
% gravity is at 0 means that the same parameters cannot quite be
% fitted exactly.  And since all our A samples have the same x scale,
% no rescaling of x is actually performed. However, for real data,
% this option should be set to 0 (the default) because then the x
% scales of each A sample are not the same (particles have an aspect
% ratio different from 1), and in changing the x scale of each A
% function we are blurring the wavelengths of the structure, and hence
% also the response of the radiation to that structure.
rescale_each_x = 1;

% x axis
x=linspace(-0.5,0.5,1024);

% Parameters describing the structure
kappa = 0.2;
beta  = 1;
fer   = 0.2; % First element ratio: zeta1 in the paper
gamma = 2;

% Mean structure: note that for kappa=0, the first cosine has an
% amplitude of 1: this is intentional as beta is intended to scale the
% rest of the sin/cos terms in the power spectrum relative to this one.
A0 = ((1+kappa./3).*cos(pi.*x)+kappa.*cos(3.*pi.*x));

% Number of sin/cos terms in the power spectrum.  Note that
% fit_ssrga_params only fits terms 2-12.
nterm=50;


J=1:nterm;

nsample=1000

clear Araw*

Araw_cat = [];

% Loop over a large number of samples
for isamp = 1:nsample

  % Generate random phases for each wavenumber
  phi = rand(1,nterm).*2.*pi;

  % Copy over the mean structure
  A = A0;

  % Initialize the expected variance
  var_expect = 0;

  % Generate structure at smaller and smaller scales by looping over
  % wavenumber
  for j = J
    % The amplitude of the sine wave at this wavenumber
    aj = sqrt(beta.*(2.*j).^-gamma);
    % Scale the first element
    if j == 1
      aj = aj*sqrt(fer);
    end
    % Create wave with random phase
    Aj = aj.*cos(2.*pi.*j.*x + phi(j));
    % Add the contribution to the variance that we expect: the
    % variance of a sine wave of amplitude 1 is 0.5
    var_expect = var_expect + 0.5.*aj.^2;
    % Add wave to total
    A = A+Aj;

    % Save first 9 waves for plotting
    if j < 10
       eval(['A' num2str(j) '=Aj;']);
    end
  end

  % Store the ratio of the true variance to the expected for this
  % sample - you can check it is close to 1 later
  var_ratio(isamp) = var(A-A0)./var_expect;

  % Add this sample to the structure: the amplitude is irrelevant
  % since it will be normalized in the fitting procedure
  Araw{isamp} = A;

end

% Fit SSRGA parameters with a prescribed "gamma"
ssrga_params = fit_ssrga_params(1,Araw,1./length(x),gamma,rescale_each_x)

% Plot the results
clf
subplot(2,1,1)
plot(x,A,'k');
hold on
plot(x,A0,'b');
plot(x,A1,'r');
plot(x,A2,'g');
legend('Sample A function','Mean structure','First wavenumber','Second wavenumber',...
      'location','northeastoutside')
xlabel('Normalized distance')
ylabel('Amplitude (arbitrary units)');

subplot(2,2,3)
plot_normalized_shape(ssrga_params);

subplot(2,2,4)
plot_spectrum(ssrga_params);

